﻿using System;
using System.IO;
using System.Text;
using System.Collections.Generic;
using Gallio.Framework;
using $safeprojectname$.Tests.Utilities;
using $projectnamespacename$.$safeprojectname$;
using $safeprojectname$.Tests.Mocks;
using System.Data;
using System.Transactions;

using MbUnit.Framework;

namespace $safeprojectname$.Tests
{
    /// <summary>
    /// Summary description for SqlDataProviderFixture
    /// </summary>
    [TestFixture]
    public class SqlDataProviderFixture
    {
        SqlDataProvider sqlDataProvider = new SqlDataProvider();

        public SqlDataProviderFixture()
        {
            var appConfig = new System.Configuration.AppSettingsReader();

            sqlDataProvider.ConnectionString = TestEnvironment.ConnectionString;
            sqlDataProvider.ObjectQualifier = TestEnvironment.ObjectQualifier;
            sqlDataProvider.DatabaseOwner = TestEnvironment.DatabaseOwner;

        }

        [SetUp]
        public void TestSetup()
        {
            // attach database
            DatabaseManager.CopyAndAttachDatabase(TestContext, TestEnvironment.DatabasePath, TestEnvironment.DatabaseName);
            // run Module installation script

        }

        [TearDown]
        public void TestTearDown()
        {
            //Detach and copy database to holding location
            DatabaseManager.DetachAndCopyDatabase(TestEnvironment.DatabaseName, TestEnvironment.DatabasePath, TestEnvironment.DatabaseName);

            //Remove Database Folder
            string testDatabaseDirectory = Path.GetFullPath(String.Format(@"{0}\{1}",
                                                        TestEnvironment.DatabasePath,
                                                        TestContext.Test.FullName));
            if (Directory.Exists(testDatabaseDirectory))
            {
                Directory.Delete(testDatabaseDirectory, true);
            }
        }

        private TestContext _testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                if(_testContextInstance==null)
                    _testContextInstance = TestContext.CurrentContext;

                return _testContextInstance;
            }
            set
            {
                _testContextInstance = value;
            }
        }

        [Test]
        [Description("Adds, retrieves and then deletes a TestDrivenModule record in the database")]
        public void AddTestDrivenModule()
        {
            string content = "";

            int itemId = sqlDataProvider.Add$safeprojectname$(MockHelper.ModuleId, MockHelper.content, MockHelper.createdByUser);
            Assert.IsTrue(itemId > 0);

            IDataReader dataReader = sqlDataProvider.Get$safeprojectname$(MockHelper.ModuleId, itemId);
            if (dataReader.Read())
            {
                content = (string)dataReader["Content"];
            }
            Assert.AreEqual<string>(MockHelper.content, content);
        }

        [Test]
        [Description("Adds, retrieves and then deletes a TestDrivenModule record in the database")]
        public void DeleteTestDrivenModule()
        {
            string content = "";

            int itemId = sqlDataProvider.Add$safeprojectname$(MockHelper.ModuleId, MockHelper.content, MockHelper.createdByUser);
            Assert.IsTrue(itemId > 0);

            IDataReader dataReader = sqlDataProvider.Get$safeprojectname$(MockHelper.ModuleId, itemId);
            if (dataReader.Read())
            {
                content = (string)dataReader["Content"];
            }
            Assert.AreEqual<string>(MockHelper.content, content);

            sqlDataProvider.Delete$safeprojectname$(MockHelper.ModuleId, itemId);

            dataReader = sqlDataProvider.Get$safeprojectname$(MockHelper.ModuleId, itemId);
            Assert.IsFalse(dataReader.Read());
        }

        [Test]
        [Description("Adds, updates and retrieves a TestDrivenModule record in the database")]
        public void Update$safeprojectname$()
        {
            string content = "";

            int itemId = sqlDataProvider.Add$safeprojectname$(MockHelper.ModuleId, MockHelper.content, MockHelper.createdByUser);
            Assert.IsTrue(itemId > 0);

            sqlDataProvider.Update$safeprojectname$(MockHelper.ModuleId, itemId, MockHelper.updateContent, MockHelper.createdByUser);

            IDataReader dataReader = sqlDataProvider.Get$safeprojectname$(MockHelper.ModuleId, itemId);
            if (dataReader.Read())
            {
                content = (string)dataReader["Content"];
            }
            Assert.AreEqual<string>(MockHelper.updateContent, content);
        }

        [Test]
        [Description("Adds 2 and then retrieves 2 TestDrivenModule records in the database")]
        public void Get$safeprojectname$s()
        {
            int records = 0;

            int itemId1 = sqlDataProvider.Add$safeprojectname$(MockHelper.ModuleId, MockHelper.content, MockHelper.createdByUser);
            int itemId2 = sqlDataProvider.Add$safeprojectname$(MockHelper.ModuleId, MockHelper.content, MockHelper.createdByUser);
            Assert.IsTrue(itemId1 > 0);
            Assert.IsTrue(itemId2 > 0);

            IDataReader dataReader = sqlDataProvider.Get$safeprojectname$s(MockHelper.ModuleId);
            while (dataReader.Read())
            {
                records += 1;
            }
            Assert.AreEqual<int>(2, records);
        }
    }
}
